@@ -6,9 +6,11 @@ import re
 import socket
 import subprocess
 import sys
+from time import sleep
 
 import pytest
 import requests
+import six
 
 import bokeh.command.subcommands.serve as scserve
 
@@ -65,6 +67,13 @@ def test_args():
             help    = "A standard Python logging format string (default: %r)" % scserve.DEFAULT_LOG_FORMAT.replace("%", "%%"),
         )),
 
+        ('--log-file', dict(
+            metavar ='LOG-FILE',
+            action  = 'store',
+            default = None,
+            help    = "A filename to write logs to, or None to write to the standard stream (default: None)",
+        )),
+
         ('files', dict(
             metavar='DIRECTORY-OR-SCRIPT',
             nargs='*',
@@ -90,13 +99,6 @@ def test_args():
             help="Public hostnames which may connect to the Bokeh websocket",
         )),
 
-        ('--host', dict(
-            metavar='HOST[:PORT]',
-            action='append',
-            type=str,
-            help="*** DEPRECATED ***",
-        )),
-
         ('--prefix', dict(
             metavar='PREFIX',
             type=str,
@@ -132,6 +134,13 @@ def test_args():
             default=None,
         )),
 
+        ('--mem-log-frequency', dict(
+            metavar='MILLISECONDS',
+            type=int,
+            help="How often to log memory usage information",
+            default=None,
+        )),
+
         ('--use-xheaders', dict(
             action='store_true',
             help="Prefer X-headers for IP/protocol information",
@@ -158,11 +167,20 @@ def test_args():
         ('--num-procs', dict(
              metavar='N',
              action='store',
-             help="Number of worker processes for an app. Default to one. Using "
-                  "0 will autodetect number of cores",
+             help="Number of worker processes for an app. Using "
+                  "0 will autodetect number of cores (defaults to 1)",
              default=1,
              type=int,
          )),
+
+         ('--websocket-max-message-size', dict(
+            metavar='BYTES',
+            action='store',
+            help="Set the Tornado websocket_max_message_size value (defaults "
+                 "to 20MB) NOTE: This setting has effect ONLY for Tornado>=4.5",
+            default=20*1024*1024,
+            type=int,
+        )),
     )
 
 
@@ -172,11 +190,16 @@ def run_bokeh_serve(args):
     p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     try:
         yield p
-    except Exception:
+    except Exception as e:
         p.terminate()
         p.wait()
-        print("---- An error occurred, subprocess stdout follows ----")
-        print(p.stdout.read().decode())
+        print("An error occurred: %s", e)
+        try:
+            out = p.stdout.read().decode()
+            print("\n---- subprocess stdout follows ----\n")
+            print(out)
+        except Exception:
+            pass
         raise
     else:
         p.terminate()
@@ -193,49 +216,6 @@ def check_error(args):
         pytest.fail("command %s unexpected successful" % (cmd,))
     return out
 
-def test__fixup_deprecated_host_args():
-    from argparse import Namespace
-
-    args = Namespace(host=None, allow_websocket_origin=None)
-    scserve._fixup_deprecated_host_args(args)
-    assert args.allow_websocket_origin == None
-
-    args = Namespace(host=[], allow_websocket_origin=None)
-    scserve._fixup_deprecated_host_args(args)
-    assert args.allow_websocket_origin == None
-
-    args = Namespace(host=[], allow_websocket_origin=[])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set([])
-
-    args = Namespace(host=['*'], allow_websocket_origin=[])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*'])
-
-    args = Namespace(host=['*'], allow_websocket_origin=['*'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*'])
-
-    args = Namespace(host=['*'], allow_websocket_origin=['*', 'foo'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*', 'foo'])
-
-    args = Namespace(host=[], allow_websocket_origin=['*'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*'])
-
-    args = Namespace(host=[], allow_websocket_origin=['*', 'foo'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*', 'foo'])
-
-    args = Namespace(host=None, allow_websocket_origin=['*'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*'])
-
-    args = Namespace(host=None, allow_websocket_origin=['*', 'foo'])
-    scserve._fixup_deprecated_host_args(args)
-    assert set(args.allow_websocket_origin) == set(['*', 'foo'])
-
 def test_host_not_available():
     host = "8.8.8.8"
     out = check_error(["--address", host])
@@ -253,18 +233,31 @@ def test_port_not_available():
     finally:
         sock.close()
 
+@pytest.mark.skipif(six.PY2, reason="Travis bug causes bad file descriptor")
 def test_actual_port_printed_out():
+    from fcntl import fcntl, F_GETFL, F_SETFL
+    from os import O_NONBLOCK, read
+    pat = re.compile(r'Bokeh app running at: http://localhost:(\d+)')
+    m = None
     with run_bokeh_serve(["--port", "0"]) as p:
-        pat = re.compile(r'Bokeh app running at: http://localhost:(\d+)')
-        while True:
-            line = p.stdout.readline()
-            print("child stdout>", line)
-            m = pat.search(line.decode())
-            if m is not None:
-                break
-        else:
+        flags = fcntl(p.stdout, F_GETFL)
+        fcntl(p.stdout, F_SETFL, flags | O_NONBLOCK)
+        sleep(2)
+        o = read(p.stdout.fileno(), 100*1024)
+        m = pat.search(o.decode())
+        if m is None:
             pytest.fail("no matching log line in process output")
         port = int(m.group(1))
         assert port > 0
         r = requests.get("http://localhost:%d/" % (port,))
         assert r.status_code == 200
+
+@pytest.mark.skipif(six.PY2, reason="Travis bug causes bad file descriptor")
+def test_websocket_max_message_size_printed_out():
+    pat = re.compile(r'Torndado websocket_max_message_size set to 12345')
+    with run_bokeh_serve(["--websocket-max-message-size", "12345"]) as p:
+        sleep(2)
+    o, e = p.communicate()
+    m = pat.search(o.decode())
+    if m is None:
+        pytest.fail("no matching log line in process output")
