@@ -11,6 +11,7 @@ from ..util.warnings import BokehUserWarning
 
 from .callbacks import Callback
 from .filters import Filter
+from .selections import Selection, SelectionPolicy, UnionRenderers
 
 pd = import_optional('pandas')
 
@@ -20,37 +21,8 @@ class DataSource(Model):
 
     '''
 
-    selected = Dict(String, Dict(String, Any), default={
-        '0d': {'glyph': None, 'indices': []},
-        '1d': {'indices': []},
-        '2d': {'indices': {}}
-    }, help="""
-    A dict to indicate selected indices on different dimensions on this DataSource. Keys are:
-
-    .. code-block:: python
-
-        # selection information for line and patch glyphs
-        '0d' : {
-          # the glyph that was selected
-          'glyph': None
-
-          # array with the [smallest] index of the segment of the line that was hit
-          'indices': []
-        }
-
-        # selection for most (point-like) glyphs, except lines and patches
-        '1d': {
-          # indices of the points included in the selection
-          indices: []
-        }
-
-        # selection information for multiline and patches glyphs
-        '2d': {
-          # mapping of indices of the multiglyph to array of glyph indices that were hit
-          # e.g. {3: [5, 6], 4: [5]}
-          indices: {}
-        }
-
+    selected = Instance(Selection, default=lambda: Selection(), help="""
+    A Selection that indicates selected indices on this DataSource.
     """)
 
     callback = Instance(Callback, help="""
@@ -64,8 +36,8 @@ class ColumnarDataSource(DataSource):
 
     '''
 
-    column_names = List(String, help="""
-    An list of names for all the columns in this DataSource.
+    selection_policy = Instance(SelectionPolicy, default=lambda: UnionRenderers(), help="""
+    An instance of a SelectionPolicy that determines how selections are set.
     """)
 
 class ColumnDataSource(ColumnarDataSource):
@@ -156,9 +128,15 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("expected a dict or pandas.DataFrame, got %s" % raw_data)
         super(ColumnDataSource, self).__init__(**kw)
-        self.column_names[:] = list(raw_data.keys())
         self.data.update(raw_data)
 
+    @property
+    def column_names(self):
+        ''' A list of the column names in this data source.
+
+        '''
+        return list(self.data)
+
     @staticmethod
     def _data_from_df(df):
         ''' Create a ``dict`` of columns from a Pandas DataFrame,
@@ -267,20 +245,13 @@ class ColumnDataSource(ColumnarDataSource):
     def to_df(self):
         ''' Convert this data source to pandas dataframe.
 
-        If ``column_names`` is set, use those. Otherwise let Pandas
-        infer the column names. The ``column_names`` property can be
-        used both to order and filter the columns.
-
         Returns:
             DataFrame
 
         '''
         if not pd:
             raise RuntimeError('Pandas must be installed to convert to a Pandas Dataframe')
-        if self.column_names:
-            return pd.DataFrame(self.data, columns=self.column_names)
-        else:
-            return pd.DataFrame(self.data)
+        return pd.DataFrame(self.data)
 
     def add(self, data, name=None):
         ''' Appends a new column of data to the data source.
@@ -299,7 +270,6 @@ class ColumnDataSource(ColumnarDataSource):
             while "Series %d"%n in self.data:
                 n += 1
             name = "Series %d"%n
-        self.column_names.append(name)
         self.data[name] = data
         return name
 
@@ -318,7 +288,6 @@ class ColumnDataSource(ColumnarDataSource):
 
         '''
         try:
-            self.column_names.remove(name)
             del self.data[name]
         except (ValueError, KeyError):
             import warnings
@@ -447,9 +416,8 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("Must stream updates to all existing columns (extra: %s)" % ", ".join(sorted(extra)))
 
+        import numpy as np
         if needs_length_check:
-            import numpy as np
-
             lengths = set()
             arr_types = (np.ndarray, pd.Series) if pd else np.ndarray
             for k, x in new_data.items():
@@ -463,10 +431,19 @@ class ColumnDataSource(ColumnarDataSource):
             if len(lengths) > 1:
                 raise ValueError("All streaming column updates must be the same length")
 
-        # slighgly awkward that we have to call converty_datetime_array here ourselves
+        # slightly awkward that we have to call convert_datetime_array here ourselves
         # but the downstream code expects things to already be ms-since-epoch
-        for key in new_data:
-            new_data[key] = convert_datetime_array(new_data[key])
+        for key, values in new_data.items():
+            if pd and isinstance(values, (pd.Series, pd.Index)):
+                values = values.values
+            old_values = self.data[key]
+            # Apply the transformation if the new data contains datetimes
+            # but the current data has already been transformed
+            if (isinstance(values, np.ndarray) and values.dtype.kind.lower() == 'm' and
+                isinstance(old_values, np.ndarray) and old_values.dtype.kind.lower() != 'm'):
+                new_data[key] = convert_datetime_array(values)
+            else:
+                new_data[key] = values
 
         self.data._stream(self.document, self, new_data, rollover, setter)
 
@@ -661,37 +638,61 @@ class GeoJSONDataSource(ColumnarDataSource):
 
 @abstract
 class RemoteSource(ColumnDataSource):
-    '''
+    ''' Base class for remote column data sources that can update from data
+    URLs at prescribed time intervals.
+
+    .. note::
+        This base class is typically not useful to instantiate on its own.
 
     '''
 
     data_url = String(help="""
-    The URL to the endpoint for the data.
+    A URL to to fetch data from.
     """)
 
     polling_interval = Int(help="""
-    polling interval for updating data source in milliseconds
+    A polling interval (in milliseconds) for updating data source.
     """)
 
 class AjaxDataSource(RemoteSource):
-    '''
+    ''' A data source that can populate columns by making Ajax calls to REST
+    enpoints.
+
+    The ``AjaxDataSource`` can be especially useful if you want to make a
+    standalone document (i.e. not backed by the Bokeh server) that can still
+    dynamically update using an existing REST API.
+
+    The response from the REST API should match the ``.data`` proeprty of a
+    standard ``ColumnDataSource``, i.e. a JSON dict that maps names to arrays
+    of values:
+
+    .. code-block:: python
+
+        {
+            'x' : [1, 2, 3, ...],
+            'y' : [9, 3, 2, ...]
+        }
+
+    A full example can be seen at :bokeh-tree:`examples/howto/ajax_source.py`
 
     '''
 
-    method = Enum('POST', 'GET', help="http method - GET or POST")
+    method = Enum('POST', 'GET', help="""
+    Specifiy the the HTTP method to use for the Ajax request (GET or POST)
+    """)
 
     mode = Enum("replace", "append", help="""
-    Whether to append new data to existing data (up to ``max_size``),
-    or to replace existing data entirely.
+    Whether to append new data to existing data (up to ``max_size``), or to
+    replace existing data entirely.
     """)
 
     max_size = Int(help="""
-    Maximum size of the data array being kept after each pull requests.
-    Larger than that size, the data will be right shifted.
+    Maximum size of the data columns. If a new fetch would result in columns
+    larger than ``max_size``, then earlier data is dropped to make room.
     """)
 
     if_modified = Bool(False, help="""
-    Whether to include an ``If-Modified-Since`` header in AJAX requests
+    Whether to include an ``If-Modified-Since`` header in Ajax requests
     to the server. If this header is supported by the server, then only
     new data since the last request will be returned.
     """)
@@ -701,5 +702,12 @@ class AjaxDataSource(RemoteSource):
     """)
 
     http_headers = Dict(String, String, help="""
-    HTTP headers to set for the Ajax request.
+    Specify HTTP headers to set for the Ajax request.
+
+    Example:
+
+    .. code-block:: python
+
+        ajax_source.headers = { 'x-my-custom-header': 'some value' }
+
     """)
