@@ -22,6 +22,8 @@ always be active regardless of what other tools are currently active.
 '''
 from __future__ import absolute_import
 
+from types import FunctionType
+
 from ..core.enums import (Anchor, Dimension, Dimensions, Location,
                           TooltipFieldFormatter, TooltipAttachment)
 from ..core.has_props import abstract
@@ -29,38 +31,30 @@ from ..core.properties import (
     Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Int, Float,
     Percent, Instance, List, Seq, String, Tuple
 )
+from ..util.compiler import nodejs_compile, CompilationError
+from ..util.dependencies import import_required
+from ..util.future import get_param_info, signature
 from ..core.validation import error
 from ..core.validation.errors import (
     INCOMPATIBLE_BOX_EDIT_RENDERER, INCOMPATIBLE_POINT_DRAW_RENDERER,
     INCOMPATIBLE_POLY_DRAW_RENDERER, INCOMPATIBLE_POLY_EDIT_RENDERER,
-    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER
+    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER, NO_RANGE_TOOL_RANGES
 )
 from ..model import Model
-from ..util.deprecation import deprecated
 
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
 from .glyphs import XYGlyph, Rect, Patches, MultiLine
+from .ranges import Range1d
 from .renderers import Renderer, GlyphRenderer
 from .layouts import LayoutDOM
 
-
 @abstract
 class Tool(Model):
     ''' A base class for all interactive tool types.
 
     '''
 
-    @property
-    def plot(self):
-        deprecated("Tool.plot property is no longer needed, and any use deprecated. In the future, accessing Tool.plot will result in an AttributeError")
-        return None
-
-    @plot.setter
-    def plot(self, val):
-        deprecated("Tool.plot property is no longer needed, and any use is deprecated. In the future, accessing Tool.plot will result in an AttributeError")
-        return None
-
 @abstract
 class Action(Tool):
     ''' A base class for tools that are buttons in the toolbar.
@@ -69,29 +63,35 @@ class Action(Tool):
     pass
 
 @abstract
-class Drag(Tool):
+class Gesture(Tool):
     ''' A base class for tools that respond to drag events.
 
     '''
     pass
 
 @abstract
-class Scroll(Tool):
+class Drag(Gesture):
+    ''' A base class for tools that respond to drag events.
+
+    '''
+    pass
+
+@abstract
+class Scroll(Gesture):
     ''' A base class for tools that respond to scroll events.
 
     '''
     pass
 
 @abstract
-class Tap(Tool):
+class Tap(Gesture):
     ''' A base class for tools that respond to tap/click events.
 
     '''
     pass
 
-
 @abstract
-class Inspection(Tool):
+class Inspection(Gesture):
     ''' A base class for tools that perform "inspections", e.g. ``HoverTool``.
 
     '''
@@ -116,7 +116,6 @@ class ToolbarBase(Model):
     A list of tools to add to the plot.
     """)
 
-
 class Toolbar(ToolbarBase):
     ''' Collect tools to display for a single plot.
 
@@ -139,6 +138,16 @@ class Toolbar(ToolbarBase):
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
+    active_multi = Instance((Gesture), help="""
+    Specify an active multi-gesture tool, for instance an edit tool or a range
+    tool.
+
+    Note that activating a multi-gesture tool will deactivate any other gesture
+    tools as appropriate. For example, if a pan tool is set as the active drag,
+    and this property is set to a ``BoxEditTool`` instance, the pan tool will
+    be deactivated (i.e. the multi-gesture tool will take precedence).
+    """)
+
 class ProxyToolbar(ToolbarBase):
     ''' A toolbar that allow to merge and proxy tools of toolbars in multiple plots. '''
 
@@ -178,6 +187,73 @@ class PanTool(Drag):
     height of the plot.
     """)
 
+DEFAULT_RANGE_OVERLAY = lambda: BoxAnnotation(
+    level="overlay",
+    render_mode="css",
+    fill_color="lightgrey",
+    fill_alpha=0.5,
+    line_color="black",
+    line_alpha=1.0,
+    line_width=0.5,
+    line_dash=[2,2],
+)
+
+class RangeTool(Drag):
+    ''' *toolbar icon*: |range_icon|
+
+    The range tool allows the user to update range objects for either or both
+    of the x- or y-dimensions by dragging a corresponding shaded annotation to
+    move it or change its boundaries.
+
+    A common use case is to add this tool to a plot with a large fixed range,
+    but to configure the tool range from a different plot. When the user
+    manipulates the overlay, the range of the second plot will be updated
+    automatically.
+
+    .. |range_icon| image:: /_images/icons/Range.png
+        :height: 18pt
+
+    '''
+
+    x_range = Instance(Range1d, help="""
+    A range synchronized to the x-dimension of the overlay. If None, the overlay
+    will span the entire x-dimension.
+    """)
+
+    x_interaction = Bool(default=True, help="""
+    Whether to respond to horizontal pan motions when an ``x_range`` is present.
+
+    By default, when an ``x_range`` is specified, it is possible to adjust the
+    horizontal position of the range box by panning horizontally inside the
+    box, or along the top or bottom edge of the box. To disable this, and fix
+    the  range box in place horizontally, set to False. (The box will still
+    update if the ``x_range`` is updated programmatically.)
+    """)
+
+    y_range = Instance(Range1d, help="""
+    A range synchronized to the y-dimension of the overlay. If None, the overlay
+    will span the entire y-dimension.
+    """)
+
+    y_interaction = Bool(default=True, help="""
+    Whether to respond to vertical pan motions when a ``y_range`` is present.
+
+    By default, when a ``y_range`` is specified, it is possible to adjust the
+    vertical position of the range box by panning vertically inside the box, or
+    along the top or bottom edge of the box. To disable this, and fix the range
+    box in place vertically, set to False. (The box will still update if the
+    ``y_range`` is updated programmatically.)
+    """)
+
+    overlay = Instance(BoxAnnotation, default=DEFAULT_RANGE_OVERLAY, help="""
+    A shaded annotation drawn to indicate the configured ranges.
+    """)
+
+    @error(NO_RANGE_TOOL_RANGES)
+    def _check_no_range_tool_ranges(self):
+        if self.x_range is None and self.y_range is None:
+            return "At least one of RangeTool.x_range or RangeTool.y_range must be configured"
+
 class WheelPanTool(Scroll):
     ''' *toolbar icon*: |wheel_pan_icon|
 
@@ -194,7 +270,6 @@ class WheelPanTool(Scroll):
     default the wheel pan tool will pan the plot along the x-axis.
     """)
 
-
 class WheelZoomTool(Scroll):
     ''' *toolbar icon*: |wheel_zoom_icon|
 
@@ -218,6 +293,21 @@ class WheelZoomTool(Scroll):
     vertically across the height of the plot.
     """)
 
+    maintain_focus = Bool(default=True, help="""
+    Whether or not zooming tool maintains its focus position. Setting it
+    to False results in a more "gliding" behavior, allowing one to
+    zoom out more smoothly, at the cost of losing the focus position.
+    """)
+
+    zoom_on_axis = Bool(default=True, help="""
+    Whether scrolling on an axis (outside the central plot area) should
+    zoom that dimension.
+    """)
+
+    speed = Float(default=1/600, help="""
+    Speed at which the wheel zooms. Default is 1/600. Optimal range is between
+    0.001 and 0.09. High values will be clipped. Speed may very between browsers.
+    """)
 
 class SaveTool(Action):
     ''' *toolbar icon*: |save_icon|
@@ -234,7 +324,6 @@ class SaveTool(Action):
 
     '''
 
-
 class ResetTool(Action):
     ''' *toolbar icon*: |reset_icon|
 
@@ -252,7 +341,6 @@ class ResetTool(Action):
 
     pass
 
-
 class TapTool(Tap):
     ''' *toolbar icon*: |tap_icon|
 
@@ -278,8 +366,8 @@ class TapTool(Tap):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -313,9 +401,6 @@ class TapTool(Tap):
 
     """)
 
-
-
-
 class CrosshairTool(Inspection):
     ''' *toolbar icon*: |crosshair_icon|
 
@@ -378,7 +463,7 @@ DEFAULT_BOX_OVERLAY = lambda: BoxAnnotation(
     line_color="black",
     line_alpha=1.0,
     line_width=2,
-    line_dash=[4, 4]
+    line_dash=[4, 4],
 )
 
 class BoxZoomTool(Drag):
@@ -418,6 +503,11 @@ class BoxZoomTool(Drag):
 
     """)
 
+    origin = Enum("corner", "center", default="corner", help="""
+    Indicates whether the rectangular zoom area should originate from a corner
+    (top-left or bottom-right depending on direction) or the center of the box.
+    """)
+
 class ZoomInTool(Action):
     ''' *toolbar icon*: |zoom_in_icon|
 
@@ -461,7 +551,6 @@ class ZoomOutTool(Action):
     Percentage to zoom for each click of the zoom-in tool.
     """)
 
-
 class BoxSelectTool(Drag):
     ''' *toolbar icon*: |box_select_icon|
 
@@ -484,8 +573,8 @@ class BoxSelectTool(Drag):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -516,6 +605,11 @@ class BoxSelectTool(Drag):
     A shaded annotation drawn to indicate the selection region.
     """)
 
+    origin = Enum("corner", "center", default="corner", help="""
+    Indicates whether the rectangular selection area should originate from a corner
+    (top-left or bottom-right depending on direction) or the center of the box.
+    """)
+
 DEFAULT_POLY_OVERLAY = lambda: PolyAnnotation(
     level="overlay",
     xs_units="screen",
@@ -555,8 +649,8 @@ class LassoSelectTool(Drag):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -577,7 +671,6 @@ class LassoSelectTool(Drag):
     A shaded annotation drawn to indicate the selection region.
     """)
 
-
 class PolySelectTool(Tap):
     ''' *toolbar icon*: |poly_select_icon|
 
@@ -606,8 +699,8 @@ class PolySelectTool(Tap):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -623,6 +716,176 @@ class PolySelectTool(Tap):
     A shaded annotation drawn to indicate the selection region.
     """)
 
+class CustomJSHover(Model):
+    ''' Define a custom formatter to apply to a hover tool field.
+
+    This model can be configured with JavaScript code to format hover tooltips.
+    The JavaScript code has access to the current value to format, some special
+    variables, and any format configured on the tooltip. The variable ``value``
+    will contain the untransformed value. The variable ``special_vars`` will
+    provide a dict with the following contents:
+
+    * ``x`` data-space x-coordinate of the mouse
+    * ``y`` data-space y-coordinate of the mouse
+    * ``sx`` screen-space x-coordinate of the mouse
+    * ``sy`` screen-space y-coordinate of the mouse
+    * ``data_x`` data-space x-coordinate of the hovered glyph
+    * ``data_y`` data-space y-coordinate of the hovered glyph
+    * ``indices`` column indices of all currently hovered glyphs
+    * ``name`` value of the ``name`` property of the hovered glyph renderer
+
+    If the hover is over a "multi" glyph such as ``Patches`` or ``MultiLine``
+    then a ``segment_index`` key will also be present.
+
+    Finally, the value of the format passed in the tooltip specification is
+    available as the ``format`` variable.
+
+    Example:
+
+        As an example, the following code adds a custom formatter to format
+        WebMercator northing coordinates (in meters) as a latitude:
+
+        .. code-block:: python
+
+            lat_custom = CustomJSHover(code="""
+                var projections = require("core/util/projections");
+                var x = special_vars.x
+                var y = special_vars.y
+                var coords = projections.wgs84_mercator.inverse([x, y])
+                return "" + coords[1]
+            """)
+
+            p.add_tools(HoverTool(
+                tooltips=[( 'lat','@y{custom}' )],
+                formatters=dict(y=lat_custom)
+            ))
+
+    .. warning::
+        The explicit purpose of this Bokeh Model is to embed *raw JavaScript
+        code* for a browser to execute. If any part of the code is derived
+        from untrusted user inputs, then you must take appropriate care to
+        sanitize the user input prior to passing to Bokeh.
+
+    '''
+
+    @classmethod
+    def from_py_func(cls, code):
+        ''' Create a CustomJSHover instance from a Python functions. The
+        function is translated to JavaScript using PScript.
+
+        The python functions must have no positional arguments. It's
+        possible to pass Bokeh models (e.g. a ColumnDataSource) as keyword
+        arguments to the functions.
+
+        The ``code`` function namespace will contain the variable ``value``
+        (the untransformed value) at render time as well as ``format`` and
+        ``special_vars`` as described in the class description.
+
+        Args:
+            code (function) : a scalar function to transform a single ``value``
+
+        Returns:
+            CustomJSHover
+
+        '''
+        if not isinstance(code, FunctionType):
+            raise ValueError('CustomJSHover.from_py_func only accepts function objects.')
+
+        pscript = import_required('pscript',
+                                  'To use Python functions for CustomJSHover, you need PScript ' +
+                                  '("conda install -c conda-forge pscript" or "pip install pscript")')
+
+        def pscript_compile(code):
+            sig = signature(code)
+
+            all_names, default_values = get_param_info(sig)
+
+            if len(all_names) - len(default_values) != 0:
+                raise ValueError("Function may only contain keyword arguments.")
+
+            if default_values and not any([isinstance(value, Model) for value in default_values]):
+                raise ValueError("Default value must be a Bokeh Model.")
+
+            func_kwargs = dict(zip(all_names, default_values))
+
+            # Wrap the code attr in a function named `code` and call it
+            # with arguments that match the `args` attr
+            code = pscript.py2js(code, 'transformer') + 'return transformer(%s);\n' % ', '.join(all_names)
+            return code, func_kwargs
+
+        jsfunc, func_kwargs = pscript_compile(code)
+
+        return cls(code=jsfunc, args=func_kwargs)
+
+    @classmethod
+    def from_coffeescript(cls, code, args={}):
+        ''' Create a CustomJSHover instance from a CoffeeScript snippet.
+        The function bodies are translated to JavaScript functions using
+        node and therefore require return statements.
+
+        The ``code`` snippet namespace will contain the variable ``value``
+        (the untransformed value) at render time as well as ``format`` and
+        ``special_vars`` as described in the class description.
+
+        Example:
+
+        .. code-block:: coffeescript
+
+            formatter = CustomJSHover.from_coffeescript("return value + " total")
+
+        Args:
+            code (str) :
+                A coffeescript snippet to transform a single ``value`` value
+
+        Returns:
+            CustomJSHover
+
+        '''
+        compiled = nodejs_compile(code, lang="coffeescript", file="???")
+        if "error" in compiled:
+            raise CompilationError(compiled.error)
+
+        return cls(code=compiled.code, args=args)
+
+    args = Dict(String, Instance(Model), help="""
+    A mapping of names to Bokeh plot objects. These objects are made
+    available to the callback code snippet as the values of named
+    parameters to the callback.
+    """)
+
+    code = String(default="", help="""
+    A snippet of JavaScript code to transform a single value. The variable
+    ``value`` will contain the untransformed value and can be expected to be
+    present in the function namespace at render time. Additionally, the
+    variable ``special_vars`` will be available, and will provide a dict
+    with the following contents:
+
+    * ``x`` data-space x-coordinate of the mouse
+    * ``y`` data-space y-coordinate of the mouse
+    * ``sx`` screen-space x-coordinate of the mouse
+    * ``sy`` screen-space y-coordinate of the mouse
+    * ``data_x`` data-space x-coordinate of the hovered glyph
+    * ``data_y`` data-space y-coordinate of the hovered glyph
+    * ``indices`` column indices of all currently hovered glyphs
+
+    If the hover is over a "multi" glyph such as ``Patches`` or ``MultiLine``
+    then a ``segment_index`` key will also be present.
+
+    Finally, the value of the format passed in the tooltip specification is
+    available as the ``format`` variable.
+
+    The snippet will be made into the body of a function and therefore requires
+    a return statement.
+
+    Example:
+
+        .. code-block:: javascript
+
+            code = '''
+            return value + " total"
+            '''
+    """)
+
 class HoverTool(Inspection):
     ''' *toolbar icon*: |crosshair_icon|
 
@@ -689,8 +952,8 @@ class HoverTool(Inspection):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), defatult="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -718,7 +981,8 @@ class HoverTool(Inspection):
 
     Field names starting with "$" are special, known fields:
 
-    :$index: index of selected point in the data source
+    :$index: index of hovered point in the data source
+    :$name: value of the ``name`` property of the hovered glyph renderer
     :$x: x-coordinate under the cursor in data space
     :$y: y-coordinate under the cursor in data space
     :$sx: x-coordinate under the cursor in screen (canvas) space
@@ -738,6 +1002,13 @@ class HoverTool(Inspection):
     surrounding it in curly braces, e.g. ``@{adjusted close}`` will display
     values from a column named ``"adjusted close"``.
 
+    Sometimes (especially with stacked charts) it is desirable to allow the
+    name of the column be specified indirectly. The field name ``@$name`` is
+    distinguished in that it will look up the ``name`` field on the hovered
+    glyph renderer, and use that value as the column name. For instance, if
+    a user hovers with the name ``"US East"``, then ``@$name`` is equivalent to
+    ``@{US East}``.
+
     By default, values for fields (e.g. ``@foo``) are displayed in a basic
     numeric format. However it is possible to control the formatting of values
     more precisely. Fields can be modified by appending a format specified to
@@ -767,7 +1038,7 @@ class HoverTool(Inspection):
 
     """).accepts(Dict(String, String), lambda d: list(d.items()))
 
-    formatters = Dict(String, Enum(TooltipFieldFormatter), default=lambda: dict(), help="""
+    formatters = Dict(String, Either(Enum(TooltipFieldFormatter), Instance(CustomJSHover)), default=lambda: dict(), help="""
     Specify the formatting scheme for data source columns, e.g.
 
     .. code-block:: python
@@ -874,7 +1145,7 @@ class RedoTool(Action):
     '''
 
 @abstract
-class EditTool(Tool):
+class EditTool(Gesture):
     ''' A base class for all interactive draw tool types.
 
     '''
@@ -909,7 +1180,9 @@ class BoxEditTool(EditTool, Drag, Tap):
 
     The supported actions include:
 
-    * Add box: Hold shift then click and drag anywhere on the plot.
+    * Add box: Hold shift then click and drag anywhere on the plot or
+      double tap once to start drawing, move the mouse and double tap
+      again to finish drawing.
 
     * Move box: Click and drag an existing box, the box will be
       dropped once you let go of the mouse button.
@@ -955,16 +1228,22 @@ class PointDrawTool(EditTool, Drag, Tap):
     ''' *toolbar icon*: |point_draw_icon|
 
     The PointDrawTool allows adding, dragging and deleting point-like
-    glyphs (of ``XYGlyph`` type) on one or more renderers by editing
-    the underlying ``ColumnDataSource`` data. Like other drawing
-    tools, the renderers that are to be edited must be supplied
-    explicitly as a list. Any newly added points will be inserted on
-    the ``ColumnDataSource`` of the first supplied renderer.
+    glyphs (of ``XYGlyph`` type) on one or more renderers by editing the
+    underlying ``ColumnDataSource`` data. Like other drawing tools, the
+    renderers that are to be edited must be supplied explicitly as a list.
+    Any newly added points will be inserted on the ``ColumnDataSource`` of
+    the first supplied renderer.
 
     The tool will automatically modify the columns on the data source
-    corresponding to the ``x`` and ``y`` values of the glyph. Any
-    additional columns in the data source will be padded with the
-    declared ``empty_value``, when adding a new point.
+    corresponding to the ``x`` and ``y`` values of the glyph. Any additional
+    columns in the data source will be padded with the given ``empty_value``
+    when adding a new point.
+
+    .. note::
+        The data source updates will trigger data change events continuously
+        throughout the edit operations on the BokehJS side. In Bokeh server
+        apps, the data source will only be synced once, when the edit operation
+        finishes.
 
     The supported actions include:
 
@@ -1042,7 +1321,6 @@ class PolyDrawTool(EditTool, Drag, Tap):
             glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
             return "%s glyph type(s) found." % glyph_types
 
-
 class PolyEditTool(EditTool, Drag, Tap):
     ''' *toolbar icon*: |poly_edit_icon|
 
