@@ -4,6 +4,7 @@ from collections import Iterable, OrderedDict, Sequence
 import difflib
 import itertools
 import re
+import textwrap
 import warnings
 
 import numpy as np
@@ -11,20 +12,166 @@ import sys
 from six import string_types, reraise
 
 from ..models import (
-    BoxSelectTool, BoxZoomTool, CategoricalAxis,
+    BoxSelectTool, BoxZoomTool, CategoricalAxis, MercatorAxis,
     TapTool, CrosshairTool, DataRange1d, DatetimeAxis,
     FactorRange, Grid, HelpTool, HoverTool, LassoSelectTool, Legend, LegendItem, LinearAxis,
     LogAxis, PanTool, ZoomInTool, ZoomOutTool, PolySelectTool, ContinuousTicker,
-    SaveTool, Range, Range1d, UndoTool, RedoTool, ResetTool, ResizeTool, Tool,
-    WheelPanTool, WheelZoomTool, ColumnarDataSource, ColumnDataSource, GlyphRenderer)
+    SaveTool, Range, Range1d, UndoTool, RedoTool, ResetTool, Tool,
+    WheelPanTool, WheelZoomTool, ColumnarDataSource, ColumnDataSource,
+    LogScale, LinearScale, CategoricalScale, Circle, MultiLine,
+    BoxEditTool, PointDrawTool, PolyDrawTool, PolyEditTool)
+from bokeh.models.markers import Marker
+from ..models.renderers import GlyphRenderer
 
 from ..core.properties import ColorSpec, Datetime, value, field
-from ..util.deprecation import deprecated
+from ..transform import stack
+from ..util.dependencies import import_optional
 from ..util.string import nice_join
 
-DEFAULT_PALETTE = ["#f22c40", "#5ab738", "#407ee7", "#df5320", "#00ad9c", "#c33ff3"]
+pd = import_optional('pandas')
 
 
+def _stack(stackers, spec0, spec1, **kw):
+    for name in (spec0, spec1):
+        if name in kw:
+            raise ValueError("Stack property '%s' cannot appear in keyword args" % name)
+
+    lengths = { len(x) for x in kw.values() if isinstance(x, (list, tuple)) }
+
+    # lengths will be empty if there are no kwargs supplied at all
+    if len(lengths) > 0:
+        if len(lengths) != 1:
+            raise ValueError("Keyword argument sequences for broadcasting must all be the same lengths. Got lengths: %r" % sorted(list(lengths)))
+        if lengths.pop() != len(stackers):
+            raise ValueError("Keyword argument sequences for broadcasting must be the same length as stackers")
+
+    s0 = []
+    s1 = []
+
+    _kw = []
+
+    for i, val in enumerate(stackers):
+        d  = {'name': val}
+        s0 = list(s1)
+        s1.append(val)
+
+        d[spec0] = stack(*s0)
+        d[spec1] = stack(*s1)
+
+        for k, v in kw.items():
+            if isinstance(v, (list, tuple)):
+                d[k] = v[i]
+            else:
+                d[k] = v
+
+        _kw.append(d)
+
+    return _kw
+
+def _graph(node_source, edge_source, **kwargs):
+
+    if not isinstance(node_source, ColumnarDataSource):
+        try:
+            # try converting the soruce to ColumnDataSource
+            node_source = ColumnDataSource(node_source)
+        except ValueError as err:
+            msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
+                curr_type=str(type(node_source)),
+                err=err.message
+            )
+            reraise(ValueError, ValueError(msg), sys.exc_info()[2])
+
+    if not isinstance(edge_source, ColumnarDataSource):
+        try:
+            # try converting the soruce to ColumnDataSource
+            edge_source = ColumnDataSource(edge_source)
+        except ValueError as err:
+            msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
+                curr_type=str(type(edge_source)),
+                err=err.message
+            )
+            reraise(ValueError, ValueError(msg), sys.exc_info()[2])
+
+    ## node stuff
+    if any(x.startswith('node_selection_') for x in kwargs):
+        snode_ca = _pop_colors_and_alpha(Circle, kwargs, prefix="node_selection_")
+    else:
+        snode_ca = None
+
+    if any(x.startswith('node_hover_') for x in kwargs):
+        hnode_ca = _pop_colors_and_alpha(Circle, kwargs, prefix="node_hover_")
+    else:
+        hnode_ca = None
+
+    if any(x.startswith('node_muted_') for x in kwargs):
+        mnode_ca = _pop_colors_and_alpha(Circle, kwargs, prefix="node_muted_")
+    else:
+        mnode_ca = None
+
+    nsnode_ca = _pop_colors_and_alpha(Circle, kwargs, prefix="node_nonselection_")
+    node_ca = _pop_colors_and_alpha(Circle, kwargs, prefix="node_")
+
+    ## edge stuff
+    if any(x.startswith('edge_selection_') for x in kwargs):
+        sedge_ca = _pop_colors_and_alpha(MultiLine, kwargs, prefix="edge_selection_")
+    else:
+        sedge_ca = None
+
+    if any(x.startswith('edge_hover_') for x in kwargs):
+        hedge_ca = _pop_colors_and_alpha(MultiLine, kwargs, prefix="edge_hover_")
+    else:
+        hedge_ca = None
+
+    if any(x.startswith('edge_muted_') for x in kwargs):
+        medge_ca = _pop_colors_and_alpha(MultiLine, kwargs, prefix="edge_muted_")
+    else:
+        medge_ca = None
+
+    nsedge_ca = _pop_colors_and_alpha(MultiLine, kwargs, prefix="edge_nonselection_")
+    edge_ca = _pop_colors_and_alpha(MultiLine, kwargs, prefix="edge_")
+
+    ## node stuff
+    node_kwargs = {k.lstrip('node_'): v for k, v in kwargs.copy().items() if k.lstrip('node_') in Circle.properties()}
+
+    node_glyph = _make_glyph(Circle, node_kwargs, node_ca)
+    nsnode_glyph = _make_glyph(Circle, node_kwargs, nsnode_ca)
+    snode_glyph = _make_glyph(Circle, node_kwargs, snode_ca)
+    hnode_glyph = _make_glyph(Circle, node_kwargs, hnode_ca)
+    mnode_glyph = _make_glyph(Circle, node_kwargs, mnode_ca)
+
+    node_renderer = GlyphRenderer(glyph=node_glyph,
+                                  nonselection_glyph=nsnode_glyph,
+                                  selection_glyph=snode_glyph,
+                                  hover_glyph=hnode_glyph,
+                                  muted_glyph=mnode_glyph,
+                                  data_source=node_source)
+
+    ## edge stuff
+    edge_kwargs = {k.lstrip('edge_'): v for k, v in kwargs.copy().items() if k.lstrip('edge_') in MultiLine.properties()}
+
+    edge_glyph = _make_glyph(MultiLine, edge_kwargs, edge_ca)
+    nsedge_glyph = _make_glyph(MultiLine, edge_kwargs, nsedge_ca)
+    sedge_glyph = _make_glyph(MultiLine, edge_kwargs, sedge_ca)
+    hedge_glyph = _make_glyph(MultiLine, edge_kwargs, hedge_ca)
+    medge_glyph = _make_glyph(MultiLine, edge_kwargs, medge_ca)
+
+    edge_renderer = GlyphRenderer(glyph=edge_glyph,
+                                  nonselection_glyph=nsedge_glyph,
+                                  selection_glyph=sedge_glyph,
+                                  hover_glyph=hedge_glyph,
+                                  muted_glyph=medge_glyph,
+                                  data_source=edge_source)
+
+    _RENDERER_ARGS = ['name', 'level', 'visible', 'x_range_name', 'y_range_name',
+                      'selection_policy', 'inspection_policy']
+
+    renderer_kwargs = {attr: kwargs.pop(attr) for attr in _RENDERER_ARGS if attr in kwargs}
+
+    renderer_kwargs["node_renderer"] = node_renderer
+    renderer_kwargs["edge_renderer"] = edge_renderer
+
+    return renderer_kwargs
+
 def get_default_color(plot=None):
     colors = [
         "#1f77b4",
@@ -47,16 +194,15 @@ def get_default_color(plot=None):
         return colors[0]
 
 
-def get_default_alpha(plot=None):
-    return 1.0
+_RENDERER_ARGS = ['name', 'x_range_name', 'y_range_name',
+                  'level', 'view', 'visible', 'muted']
 
 
 def _pop_renderer_args(kwargs):
-    result = dict(data_source=kwargs.pop('source', ColumnDataSource()))
-    for attr in ['name', 'x_range_name', 'y_range_name', 'level']:
-        val = kwargs.pop(attr, None)
-        if val:
-            result[attr] = val
+    result = {attr: kwargs.pop(attr)
+              for attr in _RENDERER_ARGS
+              if attr in kwargs}
+    result['data_source'] = kwargs.pop('source', ColumnDataSource())
     return result
 
 
@@ -101,7 +247,7 @@ def _get_legend_item_label(kwargs):
             # Do the simple thing first
             legend_item_label = value(legend)
             # But if there's a source - try and do something smart
-            if source and hasattr(source, 'column_names'):
+            if source is not None and hasattr(source, 'column_names'):
                 if legend in source.column_names:
                     legend_item_label = field(legend)
         else:
@@ -110,9 +256,18 @@ def _get_legend_item_label(kwargs):
 
 
 _GLYPH_SOURCE_MSG = """
-Supplying a user-defined data source AND iterable values to glyph methods is deprecated.
+Supplying a user-defined data source AND iterable values to glyph methods is
+not possibe. Either:
+
+Pass all data directly as literals:
+
+    p.circe(x=a_list, y=an_array, ...)
+
+Or, put all data in a ColumnDataSource and pass column names:
+
+    source = ColumnDataSource(data=dict(x=a_list, y=an_array))
+    p.circe(x='x', y='y', source=source, ...)
 
-See https://github.com/bokeh/bokeh/issues/2056 for more information.
 """
 
 
@@ -140,7 +295,7 @@ def _process_sequence_literals(glyphclass, kwargs, source, is_user_source):
             raise RuntimeError("Columns need to be 1D (%s is not)" % var)
 
         if is_user_source:
-            deprecated(_GLYPH_SOURCE_MSG)
+            raise RuntimeError(_GLYPH_SOURCE_MSG)
 
         source.add(val, name=var)
         kwargs[var] = var
@@ -187,11 +342,15 @@ def _update_legend(plot, legend_item_label, glyph_renderer):
 def _get_range(range_input):
     if range_input is None:
         return DataRange1d()
+    if pd and isinstance(range_input, pd.core.groupby.GroupBy):
+        return FactorRange(factors=sorted(list(range_input.groups.keys())))
     if isinstance(range_input, Range):
         return range_input
-    if isinstance(range_input, Sequence):
+    if pd and isinstance(range_input, pd.Series):
+        range_input = range_input.values
+    if isinstance(range_input, (Sequence, np.ndarray)):
         if all(isinstance(x, string_types) for x in range_input):
-            return FactorRange(factors=range_input)
+            return FactorRange(factors=list(range_input))
         if len(range_input) == 2:
             try:
                 return Range1d(start=range_input[0], end=range_input[1])
@@ -200,26 +359,39 @@ def _get_range(range_input):
     raise ValueError("Unrecognized range input: '%s'" % str(range_input))
 
 
-def _get_axis_class(axis_type, range_input):
+def _get_scale(range_input, axis_type):
+    if isinstance(range_input, (DataRange1d, Range1d)) and axis_type in ["linear", "datetime", "mercator", "auto", None]:
+        return LinearScale()
+    elif isinstance(range_input, (DataRange1d, Range1d)) and axis_type == "log":
+        return LogScale()
+    elif isinstance(range_input, FactorRange):
+        return CategoricalScale()
+    else:
+        raise ValueError("Unable to determine proper scale for: '%s'" % str(range_input))
+
+
+def _get_axis_class(axis_type, range_input, dim):
     if axis_type is None:
-        return None
+        return None, {}
     elif axis_type == "linear":
-        return LinearAxis
+        return LinearAxis, {}
     elif axis_type == "log":
-        return LogAxis
+        return LogAxis, {}
     elif axis_type == "datetime":
-        return DatetimeAxis
+        return DatetimeAxis, {}
+    elif axis_type == "mercator":
+        return MercatorAxis, {'dimension': 'lon' if dim == 0 else 'lat'}
     elif axis_type == "auto":
         if isinstance(range_input, FactorRange):
-            return CategoricalAxis
+            return CategoricalAxis, {}
         elif isinstance(range_input, Range1d):
             try:
                 # Easier way to validate type of Range1d parameters
                 Datetime.validate(Datetime(), range_input.start)
-                return DatetimeAxis
+                return DatetimeAxis, {}
             except ValueError:
                 pass
-        return LinearAxis
+        return LinearAxis, {}
     else:
         raise ValueError("Unrecognized axis_type: '%r'" % axis_type)
 
@@ -240,6 +412,8 @@ _known_tools = {
     "pan": lambda: PanTool(dimensions='both'),
     "xpan": lambda: PanTool(dimensions='width'),
     "ypan": lambda: PanTool(dimensions='height'),
+    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
+    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
     "wheel_zoom": lambda: WheelZoomTool(dimensions='both'),
     "xwheel_zoom": lambda: WheelZoomTool(dimensions='width'),
     "ywheel_zoom": lambda: WheelZoomTool(dimensions='height'),
@@ -249,9 +423,6 @@ _known_tools = {
     "zoom_out": lambda: ZoomOutTool(dimensions='both'),
     "xzoom_out": lambda: ZoomOutTool(dimensions='width'),
     "yzoom_out": lambda: ZoomOutTool(dimensions='height'),
-    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
-    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
-    "resize": lambda: ResizeTool(),
     "click": lambda: TapTool(behavior="inspect"),
     "tap": lambda: TapTool(),
     "crosshair": lambda: CrosshairTool(),
@@ -266,7 +437,7 @@ _known_tools = {
     "hover": lambda: HoverTool(tooltips=[
         ("index", "$index"),
         ("data (x, y)", "($x, $y)"),
-        ("canvas (x, y)", "($sx, $sy)"),
+        ("screen (x, y)", "($sx, $sy)"),
     ]),
     "save": lambda: SaveTool(),
     "previewsave": "save",
@@ -274,6 +445,10 @@ _known_tools = {
     "redo": lambda: RedoTool(),
     "reset": lambda: ResetTool(),
     "help": lambda: HelpTool(),
+    "box_edit": lambda: BoxEditTool(),
+    "point_draw": lambda: PointDrawTool(),
+    "poly_draw": lambda: PolyDrawTool(),
+    "poly_edit": lambda: PolyEditTool()
 }
 
 
@@ -298,20 +473,11 @@ def _tool_from_string(name):
 
 
 def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_label, rng, dim):
-    axiscls = _get_axis_class(axis_type, rng)
+    axiscls, axiskw = _get_axis_class(axis_type, rng, dim)
     if axiscls:
 
-        if axiscls is LogAxis:
-            # TODO (bev) this mapper type hinting is ugly
-            if dim == 0:
-                plot.x_mapper_type = 'log'
-            elif dim == 1:
-                plot.y_mapper_type = 'log'
-            else:
-                raise ValueError("received invalid dimension value: %r" % dim)
-
         # this is so we can get a ticker off the axis, even if we discard it
-        axis = axiscls(plot=plot if axis_location else None)
+        axis = axiscls(plot=plot if axis_location else None, **axiskw)
 
         if isinstance(axis.ticker, ContinuousTicker):
             axis.ticker.num_minor_ticks = _get_num_minor_ticks(axiscls, minor_ticks)
@@ -326,7 +492,7 @@ def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_lab
             getattr(plot, axis_location).append(axis)
 
 
-def _process_tools_arg(plot, tools):
+def _process_tools_arg(plot, tools, tooltips=None):
     """ Adds tools to the plot object
 
     Args:
@@ -334,6 +500,8 @@ def _process_tools_arg(plot, tools):
         tools (seq[Tool or str]|str): list of tool types or string listing the
             tool names. Those are converted using the _tool_from_string
             function. I.e.: `wheel_zoom,box_zoom,reset`.
+        tooltips (string or seq[tuple[str, str]], optional):
+            tooltips to use to configure a HoverTool
 
     Returns:
         list of Tools objects added to plot, map of supplied string names to tools
@@ -370,16 +538,25 @@ def _process_tools_arg(plot, tools):
     if repeated_tools:
         warnings.warn("%s are being repeated" % ",".join(repeated_tools))
 
+    if tooltips is not None:
+        for tool_obj in tool_objs:
+            if isinstance(tool_obj, HoverTool):
+                tool_obj.tooltips = tooltips
+                break
+        else:
+            tool_objs.append(HoverTool(tooltips=tooltips))
+
     return tool_objs, tool_map
 
 
-def _process_active_tools(toolbar, tool_map, active_drag, active_scroll, active_tap):
+def _process_active_tools(toolbar, tool_map, active_drag, active_inspect, active_scroll, active_tap):
     """ Adds tools to the plot object
 
     Args:
         toolbar (Toolbar): instance of a Toolbar object
         tools_map (dict[str]|Tool): tool_map from _process_tools_arg
         active_drag (str or Tool): the tool to set active for drag
+        active_inspect (str or Tool): the tool to set active for inspect
         active_scroll (str or Tool): the tool to set active for scroll
         active_tap (str or Tool): the tool to set active for tap
 
@@ -396,6 +573,13 @@ def _process_active_tools(toolbar, tool_map, active_drag, active_scroll, active_
     else:
         raise ValueError("Got unknown %r for 'active_drag', which was not a string supplied in 'tools' argument" % active_drag)
 
+    if active_inspect in ['auto', None] or isinstance(active_inspect, Tool) or all([isinstance(t, Tool) for t in active_inspect]):
+        toolbar.active_inspect = active_inspect
+    elif active_inspect in tool_map:
+        toolbar.active_inspect = tool_map[active_inspect]
+    else:
+        raise ValueError("Got unknown %r for 'active_inspect', which was not a string supplied in 'tools' argument" % active_scroll)
+
     if active_scroll in ['auto', None] or isinstance(active_scroll, Tool):
         toolbar.active_scroll = active_scroll
     elif active_scroll in tool_map:
@@ -414,10 +598,15 @@ def _get_argspecs(glyphclass):
     argspecs = OrderedDict()
     for arg in glyphclass._args:
         spec = {}
-        prop = getattr(glyphclass, arg)
-        spec['desc'] = " ".join(x.strip() for x in prop.__doc__.strip().split("\n\n")[0].split('\n'))
-        spec['default'] = prop.class_default(glyphclass)
-        spec['type'] = prop.__class__.__name__
+        descriptor = getattr(glyphclass, arg)
+
+        # running python with -OO will discard docstrings -> __doc__ is None
+        if descriptor.__doc__:
+            spec['desc'] = "\n        ".join(textwrap.dedent(descriptor.__doc__).split("\n"))
+        else:
+            spec['desc'] = ""
+        spec['default'] = descriptor.class_default(glyphclass)
+        spec['type'] = descriptor.property._sphinx_type()
         argspecs[arg] = spec
     return argspecs
 
@@ -454,8 +643,10 @@ def _get_sigfunc(func_name, func, argspecs):
     eval(func_code, {"func": func}, func_globals)
     return func_globals[func_name]
 
-_arg_template = "    %s (%s) : %s (default %r)"
-_doc_template = """ Configure and add %s glyphs to this Figure.
+_arg_template = """    %s (%s) : %s
+        (default: %r)
+"""
+_doc_template = """ Configure and add :class:`~bokeh.models.%s.%s` glyphs to this Figure.
 
 Args:
 %s
@@ -481,19 +672,23 @@ Returns:
 """
 
 def _add_sigfunc_info(func, argspecs, glyphclass, extra_docs):
-    func.__name__ = glyphclass.__name__.lower()
+    func.__name__ = glyphclass.__name__
+
+    omissions = {'js_event_callbacks', 'js_property_callbacks', 'subscribed_events'}
 
     kwlines = []
     kws = glyphclass.properties() - set(argspecs)
     for kw in kws:
-        prop = getattr(glyphclass, kw)
-        if prop.__doc__:
-            typ = prop.__class__.__name__
-            desc = " ".join(x.strip() for x in prop.__doc__.strip().split("\n\n")[0].split('\n'))
+        # these are not really useful, and should also really be private, just skip them
+        if kw in omissions: continue
+
+        descriptor = getattr(glyphclass, kw)
+        typ = descriptor.property._sphinx_type()
+        if descriptor.__doc__:
+            desc = "\n        ".join(textwrap.dedent(descriptor.__doc__).split("\n"))
         else:
-            typ = str(prop)
             desc = ""
-        kwlines.append(_arg_template % (kw, typ, desc, prop.class_default(glyphclass)))
+        kwlines.append(_arg_template % (kw, typ, desc, descriptor.class_default(glyphclass)))
     extra_kws = getattr(glyphclass, '_extra_kws', {})
     for kw, (typ, desc) in extra_kws.items():
         kwlines.append("    %s (%s) : %s" % (kw, typ, desc))
@@ -503,7 +698,8 @@ def _add_sigfunc_info(func, argspecs, glyphclass, extra_docs):
     for arg, spec in argspecs.items():
         arglines.append(_arg_template % (arg, spec['type'], spec['desc'], spec['default']))
 
-    func.__doc__ = _doc_template % (func.__name__, "\n".join(arglines), "\n".join(kwlines))
+    mod = "markers" if issubclass(glyphclass, Marker) else "glyphs"
+    func.__doc__ = _doc_template % (mod, func.__name__, "\n".join(arglines), "\n".join(kwlines))
     if extra_docs:
         func.__doc__ += extra_docs
 
@@ -511,26 +707,34 @@ def _glyph_function(glyphclass, extra_docs=None):
 
     def func(self, **kwargs):
 
+        # Convert data source, if necesary
+        is_user_source = kwargs.get('source', None) is not None
+        if is_user_source:
+            source = kwargs['source']
+            if not isinstance(source, ColumnarDataSource):
+                try:
+                    # try converting the soruce to ColumnDataSource
+                    source = ColumnDataSource(source)
+                except ValueError as err:
+                    msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
+                        curr_type=str(type(source)),
+                        err=err.message
+                    )
+                    reraise(ValueError, ValueError(msg), sys.exc_info()[2])
+
+                # update reddered_kws so that others can use the new source
+                kwargs['source'] = source
+
         # Process legend kwargs and remove legend before we get going
         legend_item_label = _get_legend_item_label(kwargs)
 
         # Need to check if user source is present before _pop_renderer_args
-        is_user_source = kwargs.get('source', None) is not None
         renderer_kws = _pop_renderer_args(kwargs)
         source = renderer_kws['data_source']
-        if not isinstance(source, ColumnarDataSource):
-            try:
-                # try converting the soruce to ColumnDataSource
-                source = ColumnDataSource(source)
-            except ValueError as err:
-                msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
-                    curr_type=str(type(source)),
-                    err=err.message
-                )
-                reraise(ValueError, ValueError(msg), sys.exc_info()[2])
 
-            # update reddered_kws so that others can use the new source
-            renderer_kws['data_source'] = source
+        # Assign global_alpha from alpha if glyph type is an image
+        if 'alpha' in kwargs and glyphclass.__name__ in ('Image', 'ImageRGBA', 'ImageURL'):
+            kwargs['global_alpha'] = kwargs['alpha']
 
         # handle the main glyph, need to process literals
         glyph_ca = _pop_colors_and_alpha(glyphclass, kwargs)
@@ -552,23 +756,28 @@ def _glyph_function(glyphclass, extra_docs=None):
         else:
             hglyph_ca = None
 
+        # handle the mute glyph, if any properties were given
+        if any(x.startswith('muted_') for x in kwargs):
+            mglyph_ca = _pop_colors_and_alpha(glyphclass, kwargs, prefix='muted_')
+        else:
+            mglyph_ca = None
+
         glyph = _make_glyph(glyphclass, kwargs, glyph_ca)
         nsglyph = _make_glyph(glyphclass, kwargs, nsglyph_ca)
-        hglyph = _make_glyph(glyphclass, kwargs, hglyph_ca)
         sglyph = _make_glyph(glyphclass, kwargs, sglyph_ca)
+        hglyph = _make_glyph(glyphclass, kwargs, hglyph_ca)
+        mglyph = _make_glyph(glyphclass, kwargs, mglyph_ca)
 
         glyph_renderer = GlyphRenderer(glyph=glyph,
                                        nonselection_glyph=nsglyph,
                                        selection_glyph=sglyph,
                                        hover_glyph=hglyph,
+                                       muted_glyph=mglyph,
                                        **renderer_kws)
 
         if legend_item_label:
             _update_legend(self, legend_item_label, glyph_renderer)
 
-        for tool in self.select(type=BoxSelectTool):
-            tool.renderers.append(glyph_renderer)
-
         self.renderers.append(glyph_renderer)
 
         return glyph_renderer
@@ -577,6 +786,8 @@ def _glyph_function(glyphclass, extra_docs=None):
 
     sigfunc = _get_sigfunc(glyphclass.__name__.lower(), func, argspecs)
 
+    sigfunc.glyph_method = True
+
     _add_sigfunc_info(sigfunc, argspecs, glyphclass, extra_docs)
 
     return sigfunc
